@@ -617,7 +617,7 @@ class LookerClient:
                 names are returned in the format 'explore_name.dimension_name'.
 
         """
-        logger.debug(f"Getting all dimensions from explore {explore}")
+        logger.debug(f"Getting all dimensions from explore {model}/{explore}")
         params = {"fields": ["fields"]}
         url = utils.compose_url(
             self.api_url,
@@ -840,6 +840,32 @@ class LookerClient:
         result = response.json()
         return result
 
+    def cached_lookml_validation(self, project) -> Optional[JsonDict]:
+        logger.debug(f"Getting cached LookML validation results for '{project}'")
+        url = utils.compose_url(self.api_url, path=["projects", project, "validate"])
+        response = self.get(url=url, timeout=TIMEOUT_SEC)
+
+        try:
+            response.raise_for_status()
+        except requests.exceptions.HTTPError:
+            raise LookerApiError(
+                name="unable-to-get-cached-lookml-validation",
+                title=f"Couldn't get cached LookML validation results in project '{project}'.",
+                status=response.status_code,
+                detail=(
+                    "Failed to get cached LookML valiation results. Please try again."
+                ),
+                response=response,
+            )
+
+        # If no cached validation results are available, Looker returns a 204 No Content.
+        # The response has no payload. We should return None in this case and handle accordingly.
+        if response.status_code == 204:
+            return None
+
+        result = response.json()
+        return result
+
     def all_folders(self) -> List[JsonDict]:
         logger.debug("Getting information about all folders")
         url = utils.compose_url(self.api_url, path=["folders"])
@@ -877,17 +903,22 @@ class LookerClient:
         response = self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
-            raise LookerApiError(
-                name="unable-to-retrieve-compiled-sql",
-                title="Couldn't retrieve compiled SQL.",
-                status=response.status_code,
-                detail=(
-                    f"Failed to retrieve compiled SQL for query '{query_id}'. "
-                    "Please try again."
-                ),
-                response=response,
-            )
+        except requests.exceptions.HTTPError as e:
+            if e.response.status_code == 404:
+                return (
+                    "-- SQL could not be generated because of errors with this query."
+                )
+            else:
+                raise LookerApiError(
+                    name="unable-to-retrieve-compiled-sql",
+                    title="Couldn't retrieve compiled SQL.",
+                    status=response.status_code,
+                    detail=(
+                        f"Failed to retrieve compiled SQL for query '{query_id}'. "
+                        "Please try again."
+                    ),
+                    response=response,
+                )
 
         result = response.text
 
