@@ -20,6 +20,7 @@ class Dimension(LookMlObject):
         type: str,
         tags: List[str],
         sql: str,
+        is_hidden: bool,
         url: Optional[str] = None,
     ):
         self.name = name
@@ -29,6 +30,7 @@ class Dimension(LookMlObject):
         self.tags = tags
         self.sql = sql
         self.url = url
+        self.is_hidden = is_hidden
         self.queried: bool = False
         self.errors: List[ValidationError] = []
 
@@ -78,7 +80,8 @@ class Dimension(LookMlObject):
         tags = json_dict["tags"]
         sql = json_dict["sql"]
         url = json_dict["lookml_link"]
-        return cls(name, model_name, explore_name, type, tags, sql, url)
+        is_hidden = json_dict["hidden"]
+        return cls(name, model_name, explore_name, type, tags, sql, is_hidden, url)
 
 
 class Explore(LookMlObject):
@@ -379,6 +382,7 @@ def build_dimensions(
     client: LookerClient,
     model_name: str,
     explore_name: str,
+    ignore_hidden_fields: bool = False,
 ) -> List[Dimension]:
     """Creates Dimension objects for all dimensions in a given explore."""
     dimensions_json = client.get_lookml_dimensions(model_name, explore_name)
@@ -386,7 +390,7 @@ def build_dimensions(
     for dimension_json in dimensions_json:
         dimension = Dimension.from_json(dimension_json, model_name, explore_name)
         dimension.url = client.base_url + dimension.url
-        if not dimension.ignore:
+        if not dimension.ignore and not (dimension.is_hidden and ignore_hidden_fields):
             dimensions.append(dimension)
     return dimensions
 
@@ -396,6 +400,7 @@ def build_project(
     name: str,
     filters: Optional[List[str]] = None,
     include_dimensions: bool = False,
+    ignore_hidden_fields: bool = False,
 ) -> Project:
     """Creates an object (tree) representation of a LookML project."""
     if filters is None:
@@ -428,7 +433,9 @@ def build_project(
 
         if include_dimensions:
             for explore in model.explores:
-                explore.dimensions = build_dimensions(client, model.name, explore.name)
+                explore.dimensions = build_dimensions(
+                    client, model.name, explore.name, ignore_hidden_fields
+                )
 
     project = Project(name, [model for model in models if len(model.explores) > 0])
     return project
