@@ -1,6 +1,6 @@
 import re
 from spectacles.exceptions import LookerApiError, SqlError
-from typing import List, Optional, cast, Tuple
+from typing import Dict, List, Optional, cast, Tuple
 from dataclasses import dataclass
 import itertools
 from spectacles.client import LookerClient
@@ -16,7 +16,7 @@ from spectacles.exceptions import SpectaclesException
 from spectacles.utils import time_hash
 from spectacles.lookml import build_project, Project, Explore
 from spectacles.logger import GLOBAL_LOGGER as logger
-from spectacles.printer import print_header, LINE_WIDTH
+from spectacles.printer import print_header
 
 
 @dataclass
@@ -33,30 +33,33 @@ def is_commit(string: str) -> bool:
 
 
 class LookerBranchManager:
-    def __init__(self, client: LookerClient, project: str, remote_reset: bool = False):
+    def __init__(
+        self,
+        client: LookerClient,
+        project: str,
+        remote_reset: bool = False,
+        pin_imports: Dict[str, str] = {},
+    ):
         """Context manager for Git branch checkout, creation, and deletion."""
         logger.debug(f"Setting up branch manager in project '{project}'")
         self.client = client
         self.project = project
         self.remote_reset = remote_reset
+        self.pin_imports = pin_imports
 
         state: ProjectState = self.get_project_state()
         self.workspace: str = state.workspace
         self.history: List[ProjectState] = [state]
-        self.imports: List[str] = self.get_project_imports()
-        logger.debug(
-            f"Project '{self.project}' imports the following projects: {self.imports}"
-        )
-
         self.commit: Optional[str] = None
         self.branch: Optional[str] = None
         self.is_temp_branch: bool = False
         self.import_managers: List[LookerBranchManager] = []
 
     def __call__(self, ref: Optional[str] = None, ephemeral: Optional[bool] = None):
+        logger.debug("")
         logger.debug(
-            f"\nSetting Git state for project '{self.project}' "
-            f"@ {ref or 'production'}\n" + "-" * LINE_WIDTH
+            f"Setting Git state for project '{self.project}' "
+            f"@ {ref or 'production'}:\n"
         )
         self.branch = None
         self.commit = None
@@ -88,11 +91,12 @@ class LookerBranchManager:
         return self
 
     def __enter__(self):
+        logger.indent(1)
         # A branch was passed, so we check it out in dev mode.
         if self.branch:
             self.update_workspace("dev")
             if self.ephemeral:
-                self.branch = self.checkout_temp_branch(self.branch)
+                self.branch = self.checkout_temp_branch("origin/" + self.branch)
             else:
                 self.client.checkout_branch(self.project, self.branch)
                 if self.remote_reset:
@@ -119,6 +123,11 @@ class LookerBranchManager:
             f"[ephemeral = {self.ephemeral}]"
         )
 
+        self.imports: List[str] = self.get_project_imports()
+        logger.debug(
+            f"Project '{self.project}' imports the following projects: {self.imports}"
+        )
+
         # Create temporary branches off production for manifest dependencies
         if not self.imports:
             logger.debug(f"Project '{self.project}' doesn't import any other projects")
@@ -129,11 +138,20 @@ class LookerBranchManager:
         else:
             logger.debug("Creating temporary branches in imported projects")
             for project in self.imports:
-                manager = LookerBranchManager(self.client, project)
-                manager(ephemeral=True).__enter__()
+                import_ref = self.pin_imports.get(project, None)
+                manager = LookerBranchManager(
+                    self.client, project, pin_imports=self.pin_imports
+                )
+                manager(ref=import_ref, ephemeral=True).__enter__()
                 self.import_managers.append(manager)
 
+        logger.indent(-1)
+        logger.debug("")
+
     def __exit__(self, *args):
+        logger.debug("")
+        logger.debug(f"Cleaning up Git state in '{self.project}'")
+        logger.indent(1)
         message = (
             f"Restoring project '{self.project}' to branch '{self.init_state.branch}'"
         )
@@ -155,6 +173,9 @@ class LookerBranchManager:
             self.update_workspace("dev")
             self.client.checkout_branch(self.project, self.init_state.branch)
 
+        logger.indent(-1)
+        logger.debug("")
+
     @property
     def init_state(self) -> ProjectState:
         return self.history[0]
@@ -228,10 +249,13 @@ class Runner:
         client: LookerClient,
         project: str,
         remote_reset: bool = False,
+        pin_imports: Dict[str, str] = {},
     ):
         self.project = project
         self.client = client
-        self.branch_manager = LookerBranchManager(client, project, remote_reset)
+        self.branch_manager = LookerBranchManager(
+            client, project, remote_reset, pin_imports
+        )
 
     def validate_sql(
         self,
@@ -244,6 +268,7 @@ class Runner:
         profile: bool = False,
         runtime_threshold: int = 5,
         chunk_size: int = DEFAULT_CHUNK_SIZE,
+        ignore_hidden_fields: bool = False,
     ) -> JsonDict:
         if filters is None:
             filters = ["*/*"]
@@ -256,7 +281,11 @@ class Runner:
             base_ref = self.branch_manager.ref  # Resolve the full ref after checkout
             logger.debug("Building explore tests for the desired ref")
             project = build_project(
-                self.client, name=self.project, filters=filters, include_dimensions=True
+                self.client,
+                name=self.project,
+                filters=filters,
+                include_dimensions=True,
+                ignore_hidden_fields=ignore_hidden_fields,
             )
             base_tests = validator.create_tests(
                 project, compile_sql=incremental, chunk_size=chunk_size
@@ -284,6 +313,7 @@ class Runner:
                     name=self.project,
                     filters=filters,
                     include_dimensions=True,
+                    ignore_hidden_fields=ignore_hidden_fields,
                 )
                 target_tests = validator.create_tests(
                     target_project, compile_sql=True, chunk_size=chunk_size
@@ -319,7 +349,7 @@ class Runner:
         )
 
         with self.branch_manager(ref=ref):
-            validator.run_tests(tests, profile)
+            validator.run_tests(tests, profile=profile if fail_fast else False)
 
         # Create dimension tests for the desired ref when explores errored
         if not fail_fast:
