@@ -32,7 +32,9 @@ class LookMLValidator:
 
     def validate(self, project: str, severity: str = "warning") -> Dict[str, Any]:
         severity_level: int = NAME_TO_LEVEL[severity]
-        validation_results = self.client.lookml_validation(project)
+        validation_results = self.client.cached_lookml_validation(project)
+        if not validation_results or validation_results.get("stale"):
+            validation_results = self.client.lookml_validation(project)
         errors = []
         lookml_url: Optional[str] = None
         for error in validation_results["errors"]:
